<?php declare(strict_types=1);

namespace Plugin\lfs_shopvote;

use JTL\Catalog\Product\Artikel;
use JTL\Cron\Job;
use JTL\Cron\JobInterface;
use JTL\Cron\QueueEntry;
use JTL\DB\ReturnType;
use JTL\Plugin\Helper;
use JTL\Shop;
use Plugin\lfs_shopvote\classes\LfsShopvote;

/**
 *
 */
class LfsShopvoteCron extends Job
{
    public CONST shopvoteCronRunnerFile = 'shopvote_cron.lock';

    public CONST itemsPerCall = 50;

    public CONST NumberOfReviewDays = 2;

    public function start(QueueEntry $queueEntry): JobInterface
    {
        parent::start($queueEntry);

        $logger = Shop::Container()->getLogService();

        $lfsShopvotePlugin = Helper::getPluginById('lfs_shopvote');
        $lfsShopvote = new LfsShopvote($lfsShopvotePlugin, Shop::Container()->getDB());

        $logger->debug(strtoupper($lfsShopvotePlugin->getPluginID()) . ": Cron-Aufruf");

        if (!file_exists(PFAD_ROOT . 'templates_c/' . self::shopvoteCronRunnerFile))
        {
            // Call Shopvote-API to get reviews to update
            $lfsShopvote->getArticlesToUpdate(self::NumberOfReviewDays);
            touch(PFAD_ROOT . 'templates_c/' . self::shopvoteCronRunnerFile);
            $logger->debug(strtoupper($lfsShopvotePlugin->getPluginID()) . ": Zu aktualisierende Bewertungen wurden abgerufen");
        }

        $articleReviewsToUpdate = Shop::Container()->getDB()->executeQuery(
            "SELECT * FROM xplugin_lfs_shopvote_reviewupdate_queue ORDER BY added ASC LIMIT " . self::itemsPerCall,
            ReturnType::ARRAY_OF_OBJECTS
        );

        $logger->debug(strtoupper($lfsShopvotePlugin->getPluginID()) . ": Vorhandene Queue-Einträge abgerufen");

        $finished = count($articleReviewsToUpdate)<=0;

        if (!$finished)
        {
            $logger->debug(strtoupper($lfsShopvotePlugin->getPluginID()) . ": Einträge in Queue - " . count($articleReviewsToUpdate));
            foreach ($articleReviewsToUpdate as $item) {
                $tmpArticle = new Artikel();
                $tmpArticle->fuelleArtikel((int)$item->kArtikel);
                $lfsShopvote->getProductReviews($tmpArticle);
                Shop::Container()->getDB()->deleteRow('xplugin_lfs_shopvote_reviewupdate_queue', 'kArtikel', $item->kArtikel);
            }
        }
        else
        {
            $logger->debug(strtoupper($lfsShopvotePlugin->getPluginID()) . ": 0 Einträge in Queue - Wirklich vorhandene Einträge " . count($articleReviewsToUpdate));
            $this->finishShopvoteCronRun();
            $this->setFinished($finished);
        }

        return $this;
    }

    private function finishShopvoteCronRun()
    {
        @unlink(PFAD_ROOT . 'templates_c/' . self::shopvoteCronRunnerFile);
    }


}